@@ -127,7 +127,7 @@ shared_ptr<Frame> decode_jpeg(const string &jpeg)
 		        dinfo.comp_info[0].h_samp_factor, dinfo.comp_info[0].v_samp_factor,
 		        dinfo.comp_info[1].h_samp_factor, dinfo.comp_info[1].v_samp_factor,
 		        dinfo.comp_info[2].h_samp_factor, dinfo.comp_info[2].v_samp_factor);
-		exit(1);
+		abort();
 	}
 	dinfo.raw_data_out = true;
 
@@ -238,7 +238,7 @@ shared_ptr<Frame> decode_jpeg_with_cache(FrameOnDisk frame_spec, CacheMissBehavi
 	++metric_jpeg_cache_miss_frames;
 
 	*did_decode = true;
-	shared_ptr<Frame> frame = decode_jpeg(frame_reader->read_frame(frame_spec));
+	shared_ptr<Frame> frame = decode_jpeg(frame_reader->read_frame(frame_spec, /*read_video=*/true, /*read_audio=*/false).video);
 
 	lock_guard<mutex> lock(cache_mu);
 	cache_bytes_used += frame_size(*frame);
