@@ -114,7 +114,7 @@ void Player::play_playlist_once()
 	bool clip_ready;
 	steady_clock::time_point before_sleep = steady_clock::now();
 	string pause_status;
-	float master_speed = 1.0f;
+	float master_speed = start_master_speed;
 
 	// Wait until we're supposed to play something.
 	{
@@ -274,8 +274,11 @@ void Player::play_playlist_once()
 
 				FrameOnDisk frame_lower, frame_upper;
 				bool ok = find_surrounding_frames(in_pts_secondary, secondary_stream_idx, &frame_lower, &frame_upper);
+
 				if (ok) {
 					secondary_frame = frame_lower;
+				} else {
+					secondary_stream_idx = -1;
 				}
 			}
 
@@ -355,15 +358,6 @@ void Player::play_playlist_once()
 				subtitle = ss.str();
 			}
 
-			// If there's nothing to interpolate between, or if interpolation is turned off,
-			// or we're a preview, then just display the frame.
-			if (frame_lower.pts == frame_upper.pts || global_flags.interpolation_quality == 0 || video_stream == nullptr) {
-				display_single_frame(primary_stream_idx, frame_lower, secondary_stream_idx,
-				                     secondary_frame, fade_alpha, next_frame_start, /*snapped=*/false,
-				                     subtitle, play_audio);
-				continue;
-			}
-
 			// Snap to input frame: If we can do so with less than 1% jitter
 			// (ie., move less than 1% of an _output_ frame), do so.
 			// TODO: Snap secondary (fade-to) clips in the same fashion.
@@ -383,6 +377,15 @@ void Player::play_playlist_once()
 				continue;
 			}
 
+			// If there's nothing to interpolate between, or if interpolation is turned off,
+			// or we're a preview, then just display the frame.
+			if (frame_lower.pts == frame_upper.pts || global_flags.interpolation_quality == 0 || video_stream == nullptr) {
+				display_single_frame(primary_stream_idx, frame_lower, secondary_stream_idx,
+				                     secondary_frame, fade_alpha, next_frame_start, /*snapped=*/false,
+				                     subtitle, play_audio);
+				continue;
+			}
+
 			// The snapping above makes us lock to the input framerate, even in the presence
 			// of pts drift, for most typical cases where it's needed, like converting 60 → 2x60
 			// or 60 → 2x59.94. However, there are some corner cases like 25 → 2x59.94, where we'd
@@ -487,6 +490,7 @@ void Player::display_single_frame(int primary_stream_idx, const FrameOnDisk &pri
 }
 
 // Find the frame immediately before and after this point.
+// If we have an exact match, return it immediately.
 bool Player::find_surrounding_frames(int64_t pts, int stream_idx, FrameOnDisk *frame_lower, FrameOnDisk *frame_upper)
 {
 	lock_guard<mutex> lock(frame_mu);
@@ -498,6 +502,12 @@ bool Player::find_surrounding_frames(int64_t pts, int stream_idx, FrameOnDisk *f
 	}
 	*frame_upper = *it;
 
+	// If we have an exact match, return it immediately.
+	if (frame_upper->pts == pts) {
+		*frame_lower = *it;
+		return true;
+	}
+
 	// Find the last frame such that in_pts <= frame.pts (if any).
 	if (it == frames[stream_idx].begin()) {
 		*frame_lower = *it;
@@ -621,7 +631,7 @@ TimeRemaining compute_time_left(const vector<ClipWithID> &clips, size_t currentl
 	for (size_t row = currently_playing_idx; row < clips.size(); ++row) {
 		const Clip &clip = clips[row].clip;
 		double clip_length = double(clip.pts_out - clip.pts_in) / TIMEBASE / clip.speed;
-		if (clip_length >= 86400.0) {  // More than one day.
+		if (clip_length >= 86400.0 || clip.pts_out == -1) {  // More than one day.
 			++remaining.num_infinite;
 		} else {
 			if (row == currently_playing_idx) {
