@@ -41,7 +41,7 @@ static mutex va_resources_mutex;
 #define CHECK_VASTATUS(va_status, func) \
 	if (va_status != VA_STATUS_SUCCESS) { \
 		fprintf(stderr, "%s:%d (%s) failed with %d\n", __func__, __LINE__, func, va_status); \
-		exit(1); \
+		abort(); \
 	}
 
 #define CHECK_VASTATUS_RET(va_status, func) \
