@@ -32,7 +32,7 @@ AudioEncoder::AudioEncoder(const string &codec_name, int bit_rate, const AVOutpu
 	AVCodec *codec = avcodec_find_encoder_by_name(codec_name.c_str());
 	if (codec == nullptr) {
 		fprintf(stderr, "ERROR: Could not find codec '%s'\n", codec_name.c_str());
-		exit(1);
+		abort();
 	}
 
 	ctx = avcodec_alloc_context3(codec);
@@ -47,7 +47,7 @@ AudioEncoder::AudioEncoder(const string &codec_name, int bit_rate, const AVOutpu
 	}
 	if (avcodec_open2(ctx, codec, NULL) < 0) {
 		fprintf(stderr, "Could not open codec '%s'\n", codec_name.c_str());
-		exit(1);
+		abort();
 	}
 
 	resampler = swr_alloc_set_opts(nullptr,
@@ -61,12 +61,12 @@ AudioEncoder::AudioEncoder(const string &codec_name, int bit_rate, const AVOutpu
 	                               /*log_ctx=*/nullptr);
 	if (resampler == nullptr) {
 		fprintf(stderr, "Allocating resampler failed.\n");
-		exit(1);
+		abort();
 	}
 
 	if (swr_init(resampler) < 0) {
 		fprintf(stderr, "Could not open resample context.\n");
-		exit(1);
+		abort();
 	}
 
 	audio_frame = av_frame_alloc();
@@ -116,18 +116,18 @@ void AudioEncoder::encode_audio_one_frame(const float *audio, size_t num_samples
 
 	if (av_samples_alloc(audio_frame->data, nullptr, 2, num_samples, ctx->sample_fmt, 0) < 0) {
 		fprintf(stderr, "Could not allocate %zu samples.\n", num_samples);
-		exit(1);
+		abort();
 	}
 
 	if (swr_convert(resampler, audio_frame->data, num_samples, reinterpret_cast<const uint8_t **>(&audio), num_samples) < 0) {
 		fprintf(stderr, "Audio conversion failed.\n");
-		exit(1);
+		abort();
 	}
 
 	int err = avcodec_send_frame(ctx, audio_frame);
 	if (err < 0) {
 		fprintf(stderr, "avcodec_send_frame() failed with error %d\n", err);
-		exit(1);
+		abort();
 	}
 
 	for ( ;; ) {  // Termination condition within loop.
@@ -147,7 +147,7 @@ void AudioEncoder::encode_audio_one_frame(const float *audio, size_t num_samples
 			break;
 		} else {
 			fprintf(stderr, "avcodec_receive_frame() failed with error %d\n", err);
-			exit(1);
+			abort();
 		}
 	}
 
@@ -183,7 +183,7 @@ void AudioEncoder::encode_last_audio()
 				break;
 			} else {
 				fprintf(stderr, "avcodec_receive_frame() failed with error %d\n", err);
-				exit(1);
+				abort();
 			}
 		}
 	}
