@@ -37,6 +37,7 @@ enum LongOption {
 	OPTION_HTTP_AUDIO_BITRATE,
 	OPTION_HTTP_PORT,
 	OPTION_NO_TRANSCODE_AUDIO,
+	OPTION_DISABLE_AUDIO,
 	OPTION_FLAT_AUDIO,
 	OPTION_GAIN_STAGING,
 	OPTION_DISABLE_LOCUT,
@@ -170,6 +171,7 @@ void usage(Program program)
 	if (program == PROGRAM_KAERU) {
 		fprintf(stderr, "      --no-transcode-audio        copy encoded audio raw from the source stream\n");
 		fprintf(stderr, "                                    (requires --http-audio-codec= to be set)\n");
+		fprintf(stderr, "      --disable-audio             do not include any audio in the stream\n");
 	}
 	if (program == PROGRAM_NAGERU) {
 		fprintf(stderr, "      --flat-audio                start with most audio processing turned off\n");
@@ -252,6 +254,7 @@ void parse_flags(Program program, int argc, char * const argv[])
 		{ "http-audio-bitrate", required_argument, 0, OPTION_HTTP_AUDIO_BITRATE },
 		{ "http-port", required_argument, 0, OPTION_HTTP_PORT },
 		{ "no-transcode-audio", no_argument, 0, OPTION_NO_TRANSCODE_AUDIO },
+		{ "disable-audio", no_argument, 0, OPTION_DISABLE_AUDIO },
 		{ "flat-audio", no_argument, 0, OPTION_FLAT_AUDIO },
 		{ "gain-staging", required_argument, 0, OPTION_GAIN_STAGING },
 		{ "disable-locut", no_argument, 0, OPTION_DISABLE_LOCUT },
@@ -286,7 +289,7 @@ void parse_flags(Program program, int argc, char * const argv[])
 	bool card_to_mjpeg_stream_export_set = false;
 	for ( ;; ) {
 		int option_index = 0;
-		int c = getopt_long(argc, argv, "c:t:I:r:v:m:M:w:h:", long_options, &option_index);
+		int c = getopt_long(argc, argv, "c:o:t:I:r:v:m:M:w:h:", long_options, &option_index);
 
 		if (c == -1) {
 			break;
@@ -367,6 +370,10 @@ void parse_flags(Program program, int argc, char * const argv[])
 		case OPTION_NO_TRANSCODE_AUDIO:
 			global_flags.transcode_audio = false;
 			break;
+		case OPTION_DISABLE_AUDIO:
+			global_flags.transcode_audio = false;
+			global_flags.enable_audio = false;
+			break;
 		case OPTION_HTTP_X264_VIDEO:
 			global_flags.x264_video_to_http = true;
 			break;
@@ -570,7 +577,7 @@ void parse_flags(Program program, int argc, char * const argv[])
 		fprintf(stderr, "ERROR: --output-card points to a nonexistant card\n");
 		exit(1);
 	}
-	if (!global_flags.transcode_audio && global_flags.stream_audio_codec_name.empty()) {
+	if (global_flags.enable_audio && !global_flags.transcode_audio && global_flags.stream_audio_codec_name.empty()) {
 		fprintf(stderr, "ERROR: If not transcoding audio, you must specify ahead-of-time what audio codec is in use\n");
 		fprintf(stderr, "       (using --http-audio-codec).\n");
 		exit(1);
