@@ -95,7 +95,7 @@ int main(int argc, char *argv[])
 	global_share_widget = new QGLWidget();
 	if (!global_share_widget->isValid()) {
 		fprintf(stderr, "Failed to initialize OpenGL. Nageru needs at least OpenGL 3.1 to function properly.\n");
-		exit(1);
+		abort();
 	}
 
 	MainWindow mainWindow;
@@ -120,6 +120,5 @@ int main(int argc, char *argv[])
 
 	int rc = app.exec();
 	delete global_mixer;
-	ImageInput::shutdown_updaters();
 	return rc;
 }
