@@ -198,7 +198,7 @@ MainWindow::MainWindow()
 	global_mainwindow = this;
 	ui->setupUi(this);
 
-	global_disk_space_estimator = new DiskSpaceEstimator(bind(&MainWindow::report_disk_space, this, _1, _2));
+	global_disk_space_estimator = new DiskSpaceEstimator(bind(&MainWindow::report_disk_space, this, _1, _2, _3));
 	disk_free_label = new QLabel(this);
 	disk_free_label->setStyleSheet("QLabel {padding-right: 5px;}");
 	ui->menuBar->setCornerWidget(disk_free_label);
@@ -786,7 +786,7 @@ void MainWindow::update_cutoff_labels(float cutoff_hz)
 	}
 }
 
-void MainWindow::report_disk_space(off_t free_bytes, double estimated_seconds_left)
+void MainWindow::report_disk_space(off_t free_bytes, double estimated_seconds_left, double file_length_seconds)
 {
 	char time_str[256];
 	if (estimated_seconds_left < 60.0) {
@@ -811,7 +811,9 @@ void MainWindow::report_disk_space(off_t free_bytes, double estimated_seconds_le
 	char buf[256];
 	snprintf(buf, sizeof(buf), "Disk free: %'.0f MB (approx. %s)", free_bytes / 1048576.0, time_str);
 
-	std::string label = buf;
+	// NOTE: The default formatter does not use file_length_seconds for anything,
+	// but the theme might want to do so.
+	std::string label = global_mixer->format_status_line(buf, file_length_seconds);
 
 	post_to_main_thread([this, label]{
 		disk_free_label->setText(QString::fromStdString(label));
