@@ -28,13 +28,9 @@ extern "C" {
 using namespace bmusb;
 using namespace std;
 
-extern void memcpy_with_pitch(uint8_t *dst, const uint8_t *src, size_t src_width, size_t dst_pitch, size_t height);
+static VAImageFormat uyvy_format;
 
-#define CHECK_VASTATUS(va_status, func)                                 \
-    if (va_status != VA_STATUS_SUCCESS) {                               \
-        fprintf(stderr, "%s:%d (%s) failed with %d\n", __func__, __LINE__, func, va_status); \
-        exit(1);                                                        \
-    }
+extern void memcpy_with_pitch(uint8_t *dst, const uint8_t *src, size_t src_width, size_t dst_pitch, size_t height);
 
 // From libjpeg (although it's of course identical between implementations).
 static const int jpeg_natural_order[DCTSIZE2] = {
@@ -124,7 +120,7 @@ MJPEGEncoder::MJPEGEncoder(HTTPD *httpd, const string &va_display)
 	// a situation with only one video stream (and possibly one audio stream)
 	// with known width/height, and we don't need the extra functionality it provides.
 	avctx.reset(avformat_alloc_context());
-	avctx->oformat = av_guess_format("mp4", nullptr, nullptr);
+	avctx->oformat = av_guess_format("nut", nullptr, nullptr);
 
 	uint8_t *buf = (uint8_t *)av_malloc(MUX_BUFFER_SIZE);
 	avctx->pb = avio_alloc_context(buf, MUX_BUFFER_SIZE, 1, this, nullptr, nullptr, nullptr);
@@ -135,9 +131,13 @@ MJPEGEncoder::MJPEGEncoder(HTTPD *httpd, const string &va_display)
 		AVStream *stream = avformat_new_stream(avctx.get(), nullptr);
 		if (stream == nullptr) {
 			fprintf(stderr, "avformat_new_stream() failed\n");
-			exit(1);
+			abort();
 		}
-		stream->time_base = AVRational{ 1, TIMEBASE };
+
+		// FFmpeg is very picky about having audio at 1/48000 timebase,
+		// no matter what we write. Even though we'd prefer our usual 1/120000,
+		// put the video on the same one, so that we can have locked audio.
+		stream->time_base = AVRational{ 1, OUTPUT_FREQUENCY };
 		stream->codecpar->codec_type = AVMEDIA_TYPE_VIDEO;
 		stream->codecpar->codec_id = AV_CODEC_ID_MJPEG;
 
@@ -155,6 +155,19 @@ MJPEGEncoder::MJPEGEncoder(HTTPD *httpd, const string &va_display)
 		stream->codecpar->chroma_location = AVCHROMA_LOC_LEFT;
 		stream->codecpar->field_order = AV_FIELD_PROGRESSIVE;
 	}
+	for (unsigned card_idx = 0; card_idx < global_flags.card_to_mjpeg_stream_export.size(); ++card_idx) {
+		AVStream *stream = avformat_new_stream(avctx.get(), nullptr);
+		if (stream == nullptr) {
+			fprintf(stderr, "avformat_new_stream() failed\n");
+			abort();
+		}
+		stream->time_base = AVRational{ 1, OUTPUT_FREQUENCY };
+		stream->codecpar->codec_type = AVMEDIA_TYPE_AUDIO;
+		stream->codecpar->codec_id = AV_CODEC_ID_PCM_S32LE;
+		stream->codecpar->channel_layout = AV_CH_LAYOUT_STEREO;
+		stream->codecpar->channels = 2;
+		stream->codecpar->sample_rate = OUTPUT_FREQUENCY;
+	}
 
 	AVDictionary *options = NULL;
 	vector<pair<string, string>> opts = MUX_OPTS;
@@ -163,7 +176,7 @@ MJPEGEncoder::MJPEGEncoder(HTTPD *httpd, const string &va_display)
 	}
 	if (avformat_write_header(avctx.get(), &options) < 0) {
 		fprintf(stderr, "avformat_write_header() failed\n");
-		exit(1);
+		abort();
 	}
 
 	// Initialize VA-API.
@@ -244,6 +257,7 @@ unique_ptr<VADisplayWithCleanup> MJPEGEncoder::try_open_va(const string &va_disp
 		return nullptr;
 	}
 
+	// TODO: Unify with the code in Futatabi.
 	int num_formats = vaMaxNumImageFormats(va_dpy->va_dpy);
 	assert(num_formats > 0);
 
@@ -256,10 +270,23 @@ unique_ptr<VADisplayWithCleanup> MJPEGEncoder::try_open_va(const string &va_disp
 		return nullptr;
 	}
 
+	bool found = false;
+	for (int i = 0; i < num_formats; ++i) {
+		if (formats[i].fourcc == VA_FOURCC_UYVY) {
+			memcpy(&uyvy_format, &formats[i], sizeof(VAImageFormat));
+			found = true;
+			break;
+		}
+	}
+	if (!found) {
+		if (error != nullptr) *error = "UYVY format not found";
+		return nullptr;
+	}
+
 	return va_dpy;
 }
 
-void MJPEGEncoder::upload_frame(int64_t pts, unsigned card_index, RefCountedFrame frame, const bmusb::VideoFormat &video_format, size_t y_offset, size_t cbcr_offset)
+void MJPEGEncoder::upload_frame(int64_t pts, unsigned card_index, RefCountedFrame frame, const bmusb::VideoFormat &video_format, size_t y_offset, size_t cbcr_offset, vector<int32_t> audio)
 {
 	PBOFrameAllocator::Userdata *userdata = (PBOFrameAllocator::Userdata *)frame->userdata;
 	if (video_format.width == 0 || video_format.height == 0) {
@@ -290,10 +317,25 @@ void MJPEGEncoder::upload_frame(int64_t pts, unsigned card_index, RefCountedFram
 		return;
 	}
 	++metric_mjpeg_overrun_submitted;
-	frames_to_be_encoded.push(QueuedFrame{ pts, card_index, frame, video_format, y_offset, cbcr_offset });
+	frames_to_be_encoded.push(QueuedFrame{ pts, card_index, frame, video_format, y_offset, cbcr_offset, move(audio) });
 	any_frames_to_be_encoded.notify_all();
 }
 
+int MJPEGEncoder::get_mjpeg_stream_for_card(unsigned card_index)
+{
+	// Only bother doing MJPEG encoding if there are any connected clients
+	// that want the stream.
+	if (httpd->get_num_connected_multicam_clients() == 0) {
+		return -1;
+	}
+
+	auto it = global_flags.card_to_mjpeg_stream_export.find(card_index);
+	if (it == global_flags.card_to_mjpeg_stream_export.end()) {
+		return -1;
+	}
+	return it->second;
+}
+
 void MJPEGEncoder::encoder_thread_func()
 {
 	pthread_setname_np(pthread_self(), "MJPEG_Encode");
@@ -316,9 +358,14 @@ void MJPEGEncoder::encoder_thread_func()
 			// Will call back in the receiver thread.
 			encode_jpeg_va(move(qf));
 		} else {
+			// Write audio before video, since Futatabi expects it.
+			if (qf.audio.size() > 0) {
+				write_audio_packet(qf.pts, qf.card_index, qf.audio);
+			}
+
 			// Encode synchronously, in the same thread.
 			vector<uint8_t> jpeg = encode_jpeg_libjpeg(qf);
-			write_mjpeg_packet(qf.pts, qf.card_index, jpeg);
+			write_mjpeg_packet(qf.pts, qf.card_index, jpeg.data(), jpeg.size());
 		}
 	}
 
@@ -328,21 +375,42 @@ void MJPEGEncoder::encoder_thread_func()
 	free(tmp_cr);
 }
 
-void MJPEGEncoder::write_mjpeg_packet(int64_t pts, unsigned card_index, const vector<uint8_t> &jpeg)
+void MJPEGEncoder::write_mjpeg_packet(int64_t pts, unsigned card_index, const uint8_t *jpeg, size_t jpeg_size)
 {
 	AVPacket pkt;
 	memset(&pkt, 0, sizeof(pkt));
 	pkt.buf = nullptr;
-	pkt.data = const_cast<uint8_t *>(&jpeg[0]);
-	pkt.size = jpeg.size();
+	pkt.data = const_cast<uint8_t *>(jpeg);
+	pkt.size = jpeg_size;
 	pkt.stream_index = card_index;
 	pkt.flags = AV_PKT_FLAG_KEY;
 	AVRational time_base = avctx->streams[pkt.stream_index]->time_base;
 	pkt.pts = pkt.dts = av_rescale_q(pts, AVRational{ 1, TIMEBASE }, time_base);
+	pkt.duration = 0;
 
 	if (av_write_frame(avctx.get(), &pkt) < 0) {
 		fprintf(stderr, "av_write_frame() failed\n");
-		exit(1);
+		abort();
+	}
+}
+
+void MJPEGEncoder::write_audio_packet(int64_t pts, unsigned card_index, const vector<int32_t> &audio)
+{
+	AVPacket pkt;
+	memset(&pkt, 0, sizeof(pkt));
+	pkt.buf = nullptr;
+	pkt.data = reinterpret_cast<uint8_t *>(const_cast<int32_t *>(&audio[0]));
+	pkt.size = audio.size() * sizeof(audio[0]);
+	pkt.stream_index = card_index + global_flags.card_to_mjpeg_stream_export.size();
+	pkt.flags = AV_PKT_FLAG_KEY;
+	AVRational time_base = avctx->streams[pkt.stream_index]->time_base;
+	pkt.pts = pkt.dts = av_rescale_q(pts, AVRational{ 1, TIMEBASE }, time_base);
+	size_t num_stereo_samples = audio.size() / 2;
+	pkt.duration = av_rescale_q(num_stereo_samples, AVRational{ 1, OUTPUT_FREQUENCY }, time_base);
+
+	if (av_write_frame(avctx.get(), &pkt) < 0) {
+		fprintf(stderr, "av_write_frame() failed\n");
+		abort();
 	}
 }
 
@@ -396,13 +464,16 @@ MJPEGEncoder::VAResources MJPEGEncoder::get_va_resources(unsigned width, unsigne
 	va_status = vaCreateBuffer(va_dpy->va_dpy, config_id, VAEncCodedBufferType, width * height * 3 + 8192, 1, nullptr, &ret.data_buffer);
 	CHECK_VASTATUS(va_status, "vaCreateBuffer");
 
+	va_status = vaCreateImage(va_dpy->va_dpy, &uyvy_format, width, height, &ret.image);
+	CHECK_VASTATUS(va_status, "vaCreateImage");
+
 	return ret;
 }
 
 void MJPEGEncoder::release_va_resources(MJPEGEncoder::VAResources resources)
 {
 	lock_guard<mutex> lock(va_resources_mutex);
-	if (va_resources_freelist.size() > 10) {
+	if (va_resources_freelist.size() > 50) {
 		auto it = va_resources_freelist.end();
 		--it;
 
@@ -415,6 +486,9 @@ void MJPEGEncoder::release_va_resources(MJPEGEncoder::VAResources resources)
 		va_status = vaDestroySurfaces(va_dpy->va_dpy, &it->surface, 1);
 		CHECK_VASTATUS(va_status, "vaDestroySurfaces");
 
+		va_status = vaDestroyImage(va_dpy->va_dpy, it->image.image_id);
+		CHECK_VASTATUS(va_status, "vaDestroyImage");
+
 		va_resources_freelist.erase(it);
 	}
 
@@ -597,11 +671,20 @@ MJPEGEncoder::VAData MJPEGEncoder::get_va_data_for_resolution(unsigned width, un
 
 void MJPEGEncoder::encode_jpeg_va(QueuedFrame &&qf)
 {
+	PBOFrameAllocator::Userdata *userdata = (PBOFrameAllocator::Userdata *)qf.frame->userdata;
 	unsigned width = qf.video_format.width;
 	unsigned height = qf.video_format.height;
 
-	VAResources resources = get_va_resources(width, height);
-	ReleaseVAResources release(this, resources);
+	VAResources resources;
+	ReleaseVAResources release;
+	if (userdata->data_copy_current_src == PBOFrameAllocator::Userdata::FROM_VA_API) {
+		resources = move(userdata->va_resources);
+		release = move(userdata->va_resources_release);
+	} else {
+		assert(userdata->data_copy_current_src == PBOFrameAllocator::Userdata::FROM_MALLOC);
+		resources = get_va_resources(width, height);
+		release = ReleaseVAResources(this, resources);
+	}
 
 	VAData va_data = get_va_data_for_resolution(width, height);
 	va_data.pic_param.coded_buf = resources.data_buffer;
@@ -626,27 +709,38 @@ void MJPEGEncoder::encode_jpeg_va(QueuedFrame &&qf)
 	CHECK_VASTATUS(va_status, "vaCreateBuffer");
 	VABufferDestroyer destroy_slice_param(va_dpy->va_dpy, slice_param_buffer);
 
-	VAImage image;
-	va_status = vaDeriveImage(va_dpy->va_dpy, resources.surface, &image);
-	CHECK_VASTATUS(va_status, "vaDeriveImage");
+	if (userdata->data_copy_current_src == PBOFrameAllocator::Userdata::FROM_VA_API) {
+		// The pixel data is already put into the image by the caller.
+		va_status = vaUnmapBuffer(va_dpy->va_dpy, resources.image.buf);
+		CHECK_VASTATUS(va_status, "vaUnmapBuffer");
+	} else {
+		assert(userdata->data_copy_current_src == PBOFrameAllocator::Userdata::FROM_MALLOC);
 
-	// Upload the pixel data.
-	uint8_t *surface_p = nullptr;
-	vaMapBuffer(va_dpy->va_dpy, image.buf, (void **)&surface_p);
+		// Upload the pixel data.
+		uint8_t *surface_p = nullptr;
+		vaMapBuffer(va_dpy->va_dpy, resources.image.buf, (void **)&surface_p);
 
-	size_t field_start_line = qf.video_format.extra_lines_top;  // No interlacing support.
-	size_t field_start = qf.cbcr_offset * 2 + qf.video_format.width * field_start_line * 2;
+		size_t field_start_line = qf.video_format.extra_lines_top;  // No interlacing support.
+		size_t field_start = qf.cbcr_offset * 2 + qf.video_format.width * field_start_line * 2;
 
-	{
-		const uint8_t *src = qf.frame->data_copy + field_start;
-		uint8_t *dst = (unsigned char *)surface_p + image.offsets[0];
-		memcpy_with_pitch(dst, src, qf.video_format.width * 2, image.pitches[0], qf.video_format.height);
+		{
+			const uint8_t *src = qf.frame->data_copy + field_start;
+			uint8_t *dst = (unsigned char *)surface_p + resources.image.offsets[0];
+			memcpy_with_pitch(dst, src, qf.video_format.width * 2, resources.image.pitches[0], qf.video_format.height);
+		}
+
+		va_status = vaUnmapBuffer(va_dpy->va_dpy, resources.image.buf);
+		CHECK_VASTATUS(va_status, "vaUnmapBuffer");
 	}
 
-	va_status = vaUnmapBuffer(va_dpy->va_dpy, image.buf);
-	CHECK_VASTATUS(va_status, "vaUnmapBuffer");
-	va_status = vaDestroyImage(va_dpy->va_dpy, image.image_id);
-	CHECK_VASTATUS(va_status, "vaDestroyImage");
+	qf.frame->data_copy = nullptr;
+
+	// Seemingly vaPutImage() (which triggers a GPU copy) is much nicer to the
+	// CPU than vaDeriveImage() and copying directly into the GPU's buffers.
+	// Exactly why is unclear, but it seems to involve L3 cache usage when there
+	// are many high-res (1080p+) images in play.
+	va_status = vaPutImage(va_dpy->va_dpy, resources.surface, resources.image.image_id, 0, 0, width, height, 0, 0, width, height);
+	CHECK_VASTATUS(va_status, "vaPutImage");
 
 	// Finally, stick in the JPEG header.
 	VAEncPackedHeaderParameterBuffer header_parm;
@@ -701,6 +795,11 @@ void MJPEGEncoder::va_receiver_thread_func()
 			frames_encoding.pop();
 		}
 
+		// Write audio before video, since Futatabi expects it.
+		if (qf.audio.size() > 0) {
+			write_audio_packet(qf.pts, qf.card_index, qf.audio);
+		}
+
 		VAStatus va_status = vaSyncSurface(va_dpy->va_dpy, qf.resources.surface);
 		CHECK_VASTATUS(va_status, "vaSyncSurface");
 
@@ -708,13 +807,11 @@ void MJPEGEncoder::va_receiver_thread_func()
 		va_status = vaMapBuffer(va_dpy->va_dpy, qf.resources.data_buffer, (void **)&segment);
 		CHECK_VASTATUS(va_status, "vaMapBuffer");
 
-		const char *coded_buf = reinterpret_cast<char *>(segment->buf);
-		vector<uint8_t> jpeg(coded_buf, coded_buf + segment->size);
+		const uint8_t *coded_buf = reinterpret_cast<uint8_t *>(segment->buf);
+		write_mjpeg_packet(qf.pts, qf.card_index, coded_buf, segment->size);
 
 		va_status = vaUnmapBuffer(va_dpy->va_dpy, qf.resources.data_buffer);
 		CHECK_VASTATUS(va_status, "vaUnmapBuffer");
-
-		write_mjpeg_packet(qf.pts, qf.card_index, jpeg);
 	}
 }
 
