@@ -27,12 +27,22 @@ struct jpeg_compress_struct;
 struct VADisplayWithCleanup;
 struct VectorDestinationManager;
 
+#define CHECK_VASTATUS(va_status, func)                                 \
+    if (va_status != VA_STATUS_SUCCESS) {                               \
+        fprintf(stderr, "%s:%d (%s) failed with %d\n", __func__, __LINE__, func, va_status); \
+        exit(1);                                                        \
+    }
+
 class MJPEGEncoder {
 public:
 	MJPEGEncoder(HTTPD *httpd, const std::string &va_display);
 	~MJPEGEncoder();
 	void stop();
-	void upload_frame(int64_t pts, unsigned card_index, RefCountedFrame frame, const bmusb::VideoFormat &video_format, size_t y_offset, size_t cbcr_offset);
+	void upload_frame(int64_t pts, unsigned card_index, RefCountedFrame frame, const bmusb::VideoFormat &video_format, size_t y_offset, size_t cbcr_offset, std::vector<int32_t> audio);
+	bool using_vaapi() const { return va_dpy != nullptr; }
+
+	// Returns -1 for inactive (ie., don't encode frames for this card right now).
+	int get_mjpeg_stream_for_card(unsigned card_index);
 
 private:
 	static constexpr int quality = 90;
@@ -42,6 +52,7 @@ private:
 		VASurfaceID surface;
 		VAContextID context;
 		VABufferID data_buffer;
+		VAImage image;
 	};
 
 	// RAII wrapper to release VAResources on return (even on error).
@@ -93,6 +104,7 @@ private:
 		RefCountedFrame frame;
 		bmusb::VideoFormat video_format;
 		size_t y_offset, cbcr_offset;
+		std::vector<int32_t> audio;
 
 		// Only for frames in the process of being encoded by VA-API.
 		VAResources resources;
@@ -103,7 +115,8 @@ private:
 	void va_receiver_thread_func();
 	void encode_jpeg_va(QueuedFrame &&qf);
 	std::vector<uint8_t> encode_jpeg_libjpeg(const QueuedFrame &qf);
-	void write_mjpeg_packet(int64_t pts, unsigned card_index, const std::vector<uint8_t> &jpeg);
+	void write_mjpeg_packet(int64_t pts, unsigned card_index, const uint8_t *jpeg, size_t jpeg_size);
+	void write_audio_packet(int64_t pts, unsigned card_index, const std::vector<int32_t> &audio);
 	void init_jpeg_422(unsigned width, unsigned height, VectorDestinationManager *dest, jpeg_compress_struct *cinfo);
 	std::vector<uint8_t> get_jpeg_header(unsigned width, unsigned height, jpeg_compress_struct *cinfo);
 
@@ -153,6 +166,8 @@ private:
 	std::atomic<int64_t> metric_mjpeg_frames_oversized_dropped{0};
 	std::atomic<int64_t> metric_mjpeg_overrun_dropped{0};
 	std::atomic<int64_t> metric_mjpeg_overrun_submitted{0};
+
+	friend class PBOFrameAllocator;  // FIXME
 };
 
 #endif  // !defined(_MJPEG_ENCODER_H)
