@@ -21,7 +21,7 @@ MIDIDevice::~MIDIDevice()
 	const uint64_t one = 1;
 	if (write(should_quit_fd, &one, sizeof(one)) != sizeof(one)) {
 		perror("write(should_quit_fd)");
-		exit(1);
+		abort();
 	}
 	midi_thread.join();
 	close(should_quit_fd);
